# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic.v1 import BaseModel, Field, StrictBool, constr, validator
from lusid.models.model_property import ModelProperty
from lusid.models.reconciliation_configuration import ReconciliationConfiguration
from lusid.models.reconciliation_transactions import ReconciliationTransactions
from lusid.models.resource_id import ResourceId

class CreateReconciliationRequest(BaseModel):
    """
    CreateReconciliationRequest
    """
    code: constr(strict=True, max_length=64, min_length=1) = Field(..., description="The unique identifier associated with the reconciliation")
    name: Optional[constr(strict=True, max_length=512, min_length=1)] = Field(None, description="The name of the scheduled reconciliation")
    description: Optional[constr(strict=True, max_length=1024, min_length=0)] = Field(None, description="A description of the scheduled reconciliation")
    is_portfolio_group: Optional[StrictBool] = Field(None, alias="isPortfolioGroup", description="Specifies whether reconciliation is between portfolios or portfolio groups")
    left: Optional[ResourceId] = None
    right: Optional[ResourceId] = None
    transactions: Optional[ReconciliationTransactions] = None
    positions: Optional[ReconciliationConfiguration] = None
    valuations: Optional[ReconciliationConfiguration] = None
    properties: Optional[Dict[str, ModelProperty]] = Field(None, description="Reconciliation properties")
    __properties = ["code", "name", "description", "isPortfolioGroup", "left", "right", "transactions", "positions", "valuations", "properties"]

    @validator('code')
    def code_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-zA-Z0-9\-_]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9\-_]+$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateReconciliationRequest:
        """Create an instance of CreateReconciliationRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of left
        if self.left:
            _dict['left'] = self.left.to_dict()
        # override the default output from pydantic by calling `to_dict()` of right
        if self.right:
            _dict['right'] = self.right.to_dict()
        # override the default output from pydantic by calling `to_dict()` of transactions
        if self.transactions:
            _dict['transactions'] = self.transactions.to_dict()
        # override the default output from pydantic by calling `to_dict()` of positions
        if self.positions:
            _dict['positions'] = self.positions.to_dict()
        # override the default output from pydantic by calling `to_dict()` of valuations
        if self.valuations:
            _dict['valuations'] = self.valuations.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each value in properties (dict)
        _field_dict = {}
        if self.properties:
            for _key in self.properties:
                if self.properties[_key]:
                    _field_dict[_key] = self.properties[_key].to_dict()
            _dict['properties'] = _field_dict
        # set to None if name (nullable) is None
        # and __fields_set__ contains the field
        if self.name is None and "name" in self.__fields_set__:
            _dict['name'] = None

        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        # set to None if properties (nullable) is None
        # and __fields_set__ contains the field
        if self.properties is None and "properties" in self.__fields_set__:
            _dict['properties'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateReconciliationRequest:
        """Create an instance of CreateReconciliationRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateReconciliationRequest.parse_obj(obj)

        _obj = CreateReconciliationRequest.parse_obj({
            "code": obj.get("code"),
            "name": obj.get("name"),
            "description": obj.get("description"),
            "is_portfolio_group": obj.get("isPortfolioGroup"),
            "left": ResourceId.from_dict(obj.get("left")) if obj.get("left") is not None else None,
            "right": ResourceId.from_dict(obj.get("right")) if obj.get("right") is not None else None,
            "transactions": ReconciliationTransactions.from_dict(obj.get("transactions")) if obj.get("transactions") is not None else None,
            "positions": ReconciliationConfiguration.from_dict(obj.get("positions")) if obj.get("positions") is not None else None,
            "valuations": ReconciliationConfiguration.from_dict(obj.get("valuations")) if obj.get("valuations") is not None else None,
            "properties": dict(
                (_k, ModelProperty.from_dict(_v))
                for _k, _v in obj.get("properties").items()
            )
            if obj.get("properties") is not None
            else None
        })
        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class ModelOptionsType(str, Enum):
    """
    ModelOptionsType
    """

    """
    allowed enum values
    """
    INVALID = 'Invalid'
    OPAQUEMODELOPTIONS = 'OpaqueModelOptions'
    EMPTYMODELOPTIONS = 'EmptyModelOptions'
    INDEXMODELOPTIONS = 'IndexModelOptions'
    FXFORWARDMODELOPTIONS = 'FxForwardModelOptions'
    FUNDINGLEGMODELOPTIONS = 'FundingLegModelOptions'
    EQUITYMODELOPTIONS = 'EquityModelOptions'

    @classmethod
    def from_json(cls, json_str: str) -> ModelOptionsType:
        """Create an instance of ModelOptionsType from a JSON string"""
        return ModelOptionsType(json.loads(json_str))



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, Dict
from pydantic.v1 import Field, StrictStr, constr, validator
from lusid.models.instrument_event import InstrumentEvent

class InformationalErrorEvent(InstrumentEvent):
    """
    Event holder containing error information  # noqa: E501
    """
    error_detail: constr(strict=True, min_length=1) = Field(..., alias="errorDetail", description="The details of the error")
    error_reason: constr(strict=True, min_length=1) = Field(..., alias="errorReason", description="The error reason")
    effective_at: datetime = Field(..., alias="effectiveAt", description="The effective date of the evaulation")
    instrument_event_type: StrictStr = Field(..., alias="instrumentEventType", description="The Type of Event. The available values are: TransitionEvent, InformationalEvent, OpenEvent, CloseEvent, StockSplitEvent, BondDefaultEvent, CashDividendEvent, AmortisationEvent, CashFlowEvent, ExerciseEvent, ResetEvent, TriggerEvent, RawVendorEvent, InformationalErrorEvent, BondCouponEvent, DividendReinvestmentEvent, AccumulationEvent, BondPrincipalEvent, DividendOptionEvent, MaturityEvent, FxForwardSettlementEvent, ExpiryEvent")
    additional_properties: Dict[str, Any] = {}
    __properties = ["instrumentEventType", "errorDetail", "errorReason", "effectiveAt"]

    @validator('instrument_event_type')
    def instrument_event_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('TransitionEvent', 'InformationalEvent', 'OpenEvent', 'CloseEvent', 'StockSplitEvent', 'BondDefaultEvent', 'CashDividendEvent', 'AmortisationEvent', 'CashFlowEvent', 'ExerciseEvent', 'ResetEvent', 'TriggerEvent', 'RawVendorEvent', 'InformationalErrorEvent', 'BondCouponEvent', 'DividendReinvestmentEvent', 'AccumulationEvent', 'BondPrincipalEvent', 'DividendOptionEvent', 'MaturityEvent', 'FxForwardSettlementEvent', 'ExpiryEvent'):
            raise ValueError("must be one of enum values ('TransitionEvent', 'InformationalEvent', 'OpenEvent', 'CloseEvent', 'StockSplitEvent', 'BondDefaultEvent', 'CashDividendEvent', 'AmortisationEvent', 'CashFlowEvent', 'ExerciseEvent', 'ResetEvent', 'TriggerEvent', 'RawVendorEvent', 'InformationalErrorEvent', 'BondCouponEvent', 'DividendReinvestmentEvent', 'AccumulationEvent', 'BondPrincipalEvent', 'DividendOptionEvent', 'MaturityEvent', 'FxForwardSettlementEvent', 'ExpiryEvent')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> InformationalErrorEvent:
        """Create an instance of InformationalErrorEvent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> InformationalErrorEvent:
        """Create an instance of InformationalErrorEvent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return InformationalErrorEvent.parse_obj(obj)

        _obj = InformationalErrorEvent.parse_obj({
            "instrument_event_type": obj.get("instrumentEventType"),
            "error_detail": obj.get("errorDetail"),
            "error_reason": obj.get("errorReason"),
            "effective_at": obj.get("effectiveAt")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict
from pydantic.v1 import Field, StrictBool, StrictStr, validator
from lusid.models.model_options import ModelOptions

class FxForwardModelOptions(ModelOptions):
    """
    FxForwardModelOptions
    """
    forward_rate_observable_type: StrictStr = Field(..., alias="forwardRateObservableType", description="The available values are: ForwardPoints, ForwardRate, RatesCurve, FxForwardCurve, Invalid")
    discounting_method: StrictStr = Field(..., alias="discountingMethod", description="The available values are: Standard, ConstantTimeValueOfMoney, Invalid")
    convert_to_report_ccy: StrictBool = Field(..., alias="convertToReportCcy", description="Convert all FX flows to the report currency  By setting this all FX forwards will be priced using Forward Curves that have Report Currency as the base.")
    model_options_type: StrictStr = Field(..., alias="modelOptionsType", description="The available values are: Invalid, OpaqueModelOptions, EmptyModelOptions, IndexModelOptions, FxForwardModelOptions, FundingLegModelOptions, EquityModelOptions")
    additional_properties: Dict[str, Any] = {}
    __properties = ["modelOptionsType", "forwardRateObservableType", "discountingMethod", "convertToReportCcy"]

    @validator('forward_rate_observable_type')
    def forward_rate_observable_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('ForwardPoints', 'ForwardRate', 'RatesCurve', 'FxForwardCurve', 'Invalid'):
            raise ValueError("must be one of enum values ('ForwardPoints', 'ForwardRate', 'RatesCurve', 'FxForwardCurve', 'Invalid')")
        return value

    @validator('discounting_method')
    def discounting_method_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('Standard', 'ConstantTimeValueOfMoney', 'Invalid'):
            raise ValueError("must be one of enum values ('Standard', 'ConstantTimeValueOfMoney', 'Invalid')")
        return value

    @validator('model_options_type')
    def model_options_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('Invalid', 'OpaqueModelOptions', 'EmptyModelOptions', 'IndexModelOptions', 'FxForwardModelOptions', 'FundingLegModelOptions', 'EquityModelOptions'):
            raise ValueError("must be one of enum values ('Invalid', 'OpaqueModelOptions', 'EmptyModelOptions', 'IndexModelOptions', 'FxForwardModelOptions', 'FundingLegModelOptions', 'EquityModelOptions')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FxForwardModelOptions:
        """Create an instance of FxForwardModelOptions from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> FxForwardModelOptions:
        """Create an instance of FxForwardModelOptions from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return FxForwardModelOptions.parse_obj(obj)

        _obj = FxForwardModelOptions.parse_obj({
            "model_options_type": obj.get("modelOptionsType"),
            "forward_rate_observable_type": obj.get("forwardRateObservableType"),
            "discounting_method": obj.get("discountingMethod"),
            "convert_to_report_ccy": obj.get("convertToReportCcy")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, Dict, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, constr
from lusid.models.resource_id import ResourceId
from lusid.models.version import Version

class Relation(BaseModel):
    """
    Representation of a Relation between a requested entity with the stated entity as RelationedEntityId  # noqa: E501
    """
    version: Optional[Version] = None
    relation_definition_id: ResourceId = Field(..., alias="relationDefinitionId")
    related_entity_id: Dict[str, StrictStr] = Field(..., alias="relatedEntityId")
    traversal_direction: constr(strict=True, min_length=1) = Field(..., alias="traversalDirection")
    traversal_description: constr(strict=True, min_length=1) = Field(..., alias="traversalDescription")
    effective_from: Optional[datetime] = Field(None, alias="effectiveFrom")
    __properties = ["version", "relationDefinitionId", "relatedEntityId", "traversalDirection", "traversalDescription", "effectiveFrom"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Relation:
        """Create an instance of Relation from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of version
        if self.version:
            _dict['version'] = self.version.to_dict()
        # override the default output from pydantic by calling `to_dict()` of relation_definition_id
        if self.relation_definition_id:
            _dict['relationDefinitionId'] = self.relation_definition_id.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Relation:
        """Create an instance of Relation from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Relation.parse_obj(obj)

        _obj = Relation.parse_obj({
            "version": Version.from_dict(obj.get("version")) if obj.get("version") is not None else None,
            "relation_definition_id": ResourceId.from_dict(obj.get("relationDefinitionId")) if obj.get("relationDefinitionId") is not None else None,
            "related_entity_id": obj.get("relatedEntityId"),
            "traversal_direction": obj.get("traversalDirection"),
            "traversal_description": obj.get("traversalDescription"),
            "effective_from": obj.get("effectiveFrom")
        })
        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic.v1 import BaseModel, Field
from lusid.models.dialect_id import DialectId
from lusid.models.translation_input import TranslationInput
from lusid.models.translation_script_id import TranslationScriptId

class TranslateEntitiesRequest(BaseModel):
    """
    Request to translate financial entities with a specified script stored in LUSID,  specified in the request by its id. The output of the translation is validated against a dialect stored in LUSID,  again specified in the request by its id.  # noqa: E501
    """
    entity_payloads: Dict[str, TranslationInput] = Field(..., alias="entityPayloads", description="Entity payloads to be translated, indexed by (ephemeral) unique correlation ids.")
    script_id: TranslationScriptId = Field(..., alias="scriptId")
    dialect_id: Optional[DialectId] = Field(None, alias="dialectId")
    __properties = ["entityPayloads", "scriptId", "dialectId"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TranslateEntitiesRequest:
        """Create an instance of TranslateEntitiesRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each value in entity_payloads (dict)
        _field_dict = {}
        if self.entity_payloads:
            for _key in self.entity_payloads:
                if self.entity_payloads[_key]:
                    _field_dict[_key] = self.entity_payloads[_key].to_dict()
            _dict['entityPayloads'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of script_id
        if self.script_id:
            _dict['scriptId'] = self.script_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of dialect_id
        if self.dialect_id:
            _dict['dialectId'] = self.dialect_id.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TranslateEntitiesRequest:
        """Create an instance of TranslateEntitiesRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TranslateEntitiesRequest.parse_obj(obj)

        _obj = TranslateEntitiesRequest.parse_obj({
            "entity_payloads": dict(
                (_k, TranslationInput.from_dict(_v))
                for _k, _v in obj.get("entityPayloads").items()
            )
            if obj.get("entityPayloads") is not None
            else None,
            "script_id": TranslationScriptId.from_dict(obj.get("scriptId")) if obj.get("scriptId") is not None else None,
            "dialect_id": DialectId.from_dict(obj.get("dialectId")) if obj.get("dialectId") is not None else None
        })
        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic.v1 import BaseModel, Field, StrictStr
from lusid.models.complex_market_data import ComplexMarketData
from lusid.models.complex_market_data_id import ComplexMarketDataId

class ListComplexMarketDataWithMetaDataResponse(BaseModel):
    """
    Wraps a Finbourne.WebApi.Interface.Dto.ComplexMarketData.ComplexMarketData object with information that was retrieved from storage with it.  In particular,  the scope that the data was stored in,  and a <seealso cref=\"T:Finbourne.WebApi.Interface.Dto.ComplexMarketData.ComplexMarketDataId\" /> object identifying the market data in that scope.  # noqa: E501
    """
    scope: Optional[StrictStr] = Field(None, description="The scope that the listed ComplexMarketData entity is stored in.")
    market_data_id: Optional[ComplexMarketDataId] = Field(None, alias="marketDataId")
    market_data: Optional[ComplexMarketData] = Field(None, alias="marketData")
    __properties = ["scope", "marketDataId", "marketData"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ListComplexMarketDataWithMetaDataResponse:
        """Create an instance of ListComplexMarketDataWithMetaDataResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of market_data_id
        if self.market_data_id:
            _dict['marketDataId'] = self.market_data_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of market_data
        if self.market_data:
            _dict['marketData'] = self.market_data.to_dict()
        # set to None if scope (nullable) is None
        # and __fields_set__ contains the field
        if self.scope is None and "scope" in self.__fields_set__:
            _dict['scope'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ListComplexMarketDataWithMetaDataResponse:
        """Create an instance of ListComplexMarketDataWithMetaDataResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ListComplexMarketDataWithMetaDataResponse.parse_obj(obj)

        _obj = ListComplexMarketDataWithMetaDataResponse.parse_obj({
            "scope": obj.get("scope"),
            "market_data_id": ComplexMarketDataId.from_dict(obj.get("marketDataId")) if obj.get("marketDataId") is not None else None,
            "market_data": ComplexMarketData.from_dict(obj.get("marketData")) if obj.get("marketData") is not None else None
        })
        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict
from pydantic.v1 import BaseModel, Field, constr, validator
from lusid.models.trade_ticket import TradeTicket

class TranslateTradeTicketRequest(BaseModel):
    """
    A collection of instruments to translate, along with the target dialect to translate into.  # noqa: E501
    """
    tickets: Dict[str, TradeTicket] = Field(..., description="The collection of trade tickets to translate.                Each trade ticket should be keyed by a unique correlation id. This id is ephemeral  and is not stored by LUSID. It serves only as a way to easily identify each instrument in the response.")
    dialect: constr(strict=True, max_length=256, min_length=1) = Field(..., description="The target dialect that the given instruments should be translated to.")
    __properties = ["tickets", "dialect"]

    @validator('dialect')
    def dialect_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-zA-Z]*$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z]*$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TranslateTradeTicketRequest:
        """Create an instance of TranslateTradeTicketRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each value in tickets (dict)
        _field_dict = {}
        if self.tickets:
            for _key in self.tickets:
                if self.tickets[_key]:
                    _field_dict[_key] = self.tickets[_key].to_dict()
            _dict['tickets'] = _field_dict
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TranslateTradeTicketRequest:
        """Create an instance of TranslateTradeTicketRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TranslateTradeTicketRequest.parse_obj(obj)

        _obj = TranslateTradeTicketRequest.parse_obj({
            "tickets": dict(
                (_k, TradeTicket.from_dict(_v))
                for _k, _v in obj.get("tickets").items()
            )
            if obj.get("tickets") is not None
            else None,
            "dialect": obj.get("dialect")
        })
        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, Dict, List, Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr, conlist
from lusid.models.link import Link
from lusid.models.resource_id import ResourceId

class Change(BaseModel):
    """
    The time an entity was modified (amendment and/or historical correction).  # noqa: E501
    """
    href: Optional[StrictStr] = None
    entity_id: ResourceId = Field(..., alias="entityId")
    corrected: StrictBool = Field(...)
    correction_effective_at: Optional[datetime] = Field(None, alias="correctionEffectiveAt")
    correction_as_at: Optional[datetime] = Field(None, alias="correctionAsAt")
    amended: StrictBool = Field(...)
    amendment_effective_at: Optional[datetime] = Field(None, alias="amendmentEffectiveAt")
    amendment_as_at: Optional[datetime] = Field(None, alias="amendmentAsAt")
    links: Optional[conlist(Link)] = None
    __properties = ["href", "entityId", "corrected", "correctionEffectiveAt", "correctionAsAt", "amended", "amendmentEffectiveAt", "amendmentAsAt", "links"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Change:
        """Create an instance of Change from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of entity_id
        if self.entity_id:
            _dict['entityId'] = self.entity_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item in self.links:
                if _item:
                    _items.append(_item.to_dict())
            _dict['links'] = _items
        # set to None if href (nullable) is None
        # and __fields_set__ contains the field
        if self.href is None and "href" in self.__fields_set__:
            _dict['href'] = None

        # set to None if correction_effective_at (nullable) is None
        # and __fields_set__ contains the field
        if self.correction_effective_at is None and "correction_effective_at" in self.__fields_set__:
            _dict['correctionEffectiveAt'] = None

        # set to None if correction_as_at (nullable) is None
        # and __fields_set__ contains the field
        if self.correction_as_at is None and "correction_as_at" in self.__fields_set__:
            _dict['correctionAsAt'] = None

        # set to None if amendment_effective_at (nullable) is None
        # and __fields_set__ contains the field
        if self.amendment_effective_at is None and "amendment_effective_at" in self.__fields_set__:
            _dict['amendmentEffectiveAt'] = None

        # set to None if amendment_as_at (nullable) is None
        # and __fields_set__ contains the field
        if self.amendment_as_at is None and "amendment_as_at" in self.__fields_set__:
            _dict['amendmentAsAt'] = None

        # set to None if links (nullable) is None
        # and __fields_set__ contains the field
        if self.links is None and "links" in self.__fields_set__:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Change:
        """Create an instance of Change from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Change.parse_obj(obj)

        _obj = Change.parse_obj({
            "href": obj.get("href"),
            "entity_id": ResourceId.from_dict(obj.get("entityId")) if obj.get("entityId") is not None else None,
            "corrected": obj.get("corrected"),
            "correction_effective_at": obj.get("correctionEffectiveAt"),
            "correction_as_at": obj.get("correctionAsAt"),
            "amended": obj.get("amended"),
            "amendment_effective_at": obj.get("amendmentEffectiveAt"),
            "amendment_as_at": obj.get("amendmentAsAt"),
            "links": [Link.from_dict(_item) for _item in obj.get("links")] if obj.get("links") is not None else None
        })
        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List
from pydantic.v1 import BaseModel, Field, conlist
from lusid.models.field_definition import FieldDefinition
from lusid.models.field_value import FieldValue

class ReferenceData(BaseModel):
    """
    ReferenceData
    """
    field_definitions: conlist(FieldDefinition) = Field(..., alias="fieldDefinitions")
    values: conlist(FieldValue) = Field(...)
    __properties = ["fieldDefinitions", "values"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ReferenceData:
        """Create an instance of ReferenceData from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in field_definitions (list)
        _items = []
        if self.field_definitions:
            for _item in self.field_definitions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['fieldDefinitions'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in values (list)
        _items = []
        if self.values:
            for _item in self.values:
                if _item:
                    _items.append(_item.to_dict())
            _dict['values'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ReferenceData:
        """Create an instance of ReferenceData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ReferenceData.parse_obj(obj)

        _obj = ReferenceData.parse_obj({
            "field_definitions": [FieldDefinition.from_dict(_item) for _item in obj.get("fieldDefinitions")] if obj.get("fieldDefinitions") is not None else None,
            "values": [FieldValue.from_dict(_item) for _item in obj.get("values")] if obj.get("values") is not None else None
        })
        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Union
from pydantic.v1 import Field, StrictBool, StrictFloat, StrictInt, StrictStr, constr, validator
from lusid.models.floating_leg import FloatingLeg
from lusid.models.lusid_instrument import LusidInstrument

class CapFloor(LusidInstrument):
    """
    LUSID representation of Cap, Floor, or Collar.  # noqa: E501
    """
    cap_floor_type: constr(strict=True, min_length=1) = Field(..., alias="capFloorType", description="Determine if it's CAP, FLOOR, or COLLAR.    Supported string (enumeration) values are: [Cap, Floor, Collar].")
    cap_strike: Union[StrictFloat, StrictInt] = Field(..., alias="capStrike", description="Strike rate of the Cap.")
    floor_strike: Union[StrictFloat, StrictInt] = Field(..., alias="floorStrike", description="Strike rate of the Floor.")
    include_first_caplet: StrictBool = Field(..., alias="includeFirstCaplet", description="Include first caplet flag.")
    underlying_floating_leg: FloatingLeg = Field(..., alias="underlyingFloatingLeg")
    instrument_type: StrictStr = Field(..., alias="instrumentType", description="The available values are: QuotedSecurity, InterestRateSwap, FxForward, Future, ExoticInstrument, FxOption, CreditDefaultSwap, InterestRateSwaption, Bond, EquityOption, FixedLeg, FloatingLeg, BespokeCashFlowsLeg, Unknown, TermDeposit, ContractForDifference, EquitySwap, CashPerpetual, CapFloor, CashSettled, CdsIndex, Basket, FundingLeg, FxSwap, ForwardRateAgreement, SimpleInstrument, Repo, Equity, ExchangeTradedOption, ReferenceInstrument, ComplexBond, InflationLinkedBond, InflationSwap, SimpleCashFlowLoan, TotalReturnSwap, InflationLeg, FundShareClass, FlexibleLoan")
    additional_properties: Dict[str, Any] = {}
    __properties = ["instrumentType", "capFloorType", "capStrike", "floorStrike", "includeFirstCaplet", "underlyingFloatingLeg"]

    @validator('instrument_type')
    def instrument_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('QuotedSecurity', 'InterestRateSwap', 'FxForward', 'Future', 'ExoticInstrument', 'FxOption', 'CreditDefaultSwap', 'InterestRateSwaption', 'Bond', 'EquityOption', 'FixedLeg', 'FloatingLeg', 'BespokeCashFlowsLeg', 'Unknown', 'TermDeposit', 'ContractForDifference', 'EquitySwap', 'CashPerpetual', 'CapFloor', 'CashSettled', 'CdsIndex', 'Basket', 'FundingLeg', 'FxSwap', 'ForwardRateAgreement', 'SimpleInstrument', 'Repo', 'Equity', 'ExchangeTradedOption', 'ReferenceInstrument', 'ComplexBond', 'InflationLinkedBond', 'InflationSwap', 'SimpleCashFlowLoan', 'TotalReturnSwap', 'InflationLeg', 'FundShareClass', 'FlexibleLoan'):
            raise ValueError("must be one of enum values ('QuotedSecurity', 'InterestRateSwap', 'FxForward', 'Future', 'ExoticInstrument', 'FxOption', 'CreditDefaultSwap', 'InterestRateSwaption', 'Bond', 'EquityOption', 'FixedLeg', 'FloatingLeg', 'BespokeCashFlowsLeg', 'Unknown', 'TermDeposit', 'ContractForDifference', 'EquitySwap', 'CashPerpetual', 'CapFloor', 'CashSettled', 'CdsIndex', 'Basket', 'FundingLeg', 'FxSwap', 'ForwardRateAgreement', 'SimpleInstrument', 'Repo', 'Equity', 'ExchangeTradedOption', 'ReferenceInstrument', 'ComplexBond', 'InflationLinkedBond', 'InflationSwap', 'SimpleCashFlowLoan', 'TotalReturnSwap', 'InflationLeg', 'FundShareClass', 'FlexibleLoan')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CapFloor:
        """Create an instance of CapFloor from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of underlying_floating_leg
        if self.underlying_floating_leg:
            _dict['underlyingFloatingLeg'] = self.underlying_floating_leg.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CapFloor:
        """Create an instance of CapFloor from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CapFloor.parse_obj(obj)

        _obj = CapFloor.parse_obj({
            "instrument_type": obj.get("instrumentType"),
            "cap_floor_type": obj.get("capFloorType"),
            "cap_strike": obj.get("capStrike"),
            "floor_strike": obj.get("floorStrike"),
            "include_first_caplet": obj.get("includeFirstCaplet"),
            "underlying_floating_leg": FloatingLeg.from_dict(obj.get("underlyingFloatingLeg")) if obj.get("underlyingFloatingLeg") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class PeriodType(str, Enum):
    """
    PeriodType
    """

    """
    allowed enum values
    """
    DAILY = 'Daily'
    WEEKLY = 'Weekly'
    MONTHLY = 'Monthly'
    QUARTERLY = 'Quarterly'
    ANNUALLY = 'Annually'

    @classmethod
    def from_json(cls, json_str: str) -> PeriodType:
        """Create an instance of PeriodType from a JSON string"""
        return PeriodType(json.loads(json_str))



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List
from pydantic.v1 import Field, StrictStr, conlist, constr, validator
from lusid.models.compliance_step import ComplianceStep
from lusid.models.compliance_template_parameter import ComplianceTemplateParameter

class GroupFilterStep(ComplianceStep):
    """
    GroupFilterStep
    """
    label: constr(strict=True, min_length=1) = Field(..., description="The label of the compliance step")
    limit_check_parameters: conlist(ComplianceTemplateParameter) = Field(..., alias="limitCheckParameters", description="Parameters required for an absolute limit check")
    warning_check_parameters: conlist(ComplianceTemplateParameter) = Field(..., alias="warningCheckParameters", description="Parameters required for a warning limit check")
    compliance_step_type: StrictStr = Field(..., alias="complianceStepType", description=". The available values are: FilterStep, GroupByStep, GroupFilterStep, BranchStep, RecombineStep, CheckStep")
    additional_properties: Dict[str, Any] = {}
    __properties = ["complianceStepType", "label", "limitCheckParameters", "warningCheckParameters"]

    @validator('compliance_step_type')
    def compliance_step_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('FilterStep', 'GroupByStep', 'GroupFilterStep', 'BranchStep', 'RecombineStep', 'CheckStep'):
            raise ValueError("must be one of enum values ('FilterStep', 'GroupByStep', 'GroupFilterStep', 'BranchStep', 'RecombineStep', 'CheckStep')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GroupFilterStep:
        """Create an instance of GroupFilterStep from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in limit_check_parameters (list)
        _items = []
        if self.limit_check_parameters:
            for _item in self.limit_check_parameters:
                if _item:
                    _items.append(_item.to_dict())
            _dict['limitCheckParameters'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in warning_check_parameters (list)
        _items = []
        if self.warning_check_parameters:
            for _item in self.warning_check_parameters:
                if _item:
                    _items.append(_item.to_dict())
            _dict['warningCheckParameters'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GroupFilterStep:
        """Create an instance of GroupFilterStep from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GroupFilterStep.parse_obj(obj)

        _obj = GroupFilterStep.parse_obj({
            "compliance_step_type": obj.get("complianceStepType"),
            "label": obj.get("label"),
            "limit_check_parameters": [ComplianceTemplateParameter.from_dict(_item) for _item in obj.get("limitCheckParameters")] if obj.get("limitCheckParameters") is not None else None,
            "warning_check_parameters": [ComplianceTemplateParameter.from_dict(_item) for _item in obj.get("warningCheckParameters")] if obj.get("warningCheckParameters") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic.v1 import BaseModel, Field, conlist
from lusid.models.mapping import Mapping
from lusid.models.reconciled_transaction import ReconciledTransaction

class TransactionsReconciliationsResponse(BaseModel):
    """
    TransactionsReconciliationsResponse
    """
    mapping: Optional[Mapping] = None
    data: Optional[conlist(ReconciledTransaction)] = Field(None, description="The result of this reconciliation")
    __properties = ["mapping", "data"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TransactionsReconciliationsResponse:
        """Create an instance of TransactionsReconciliationsResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of mapping
        if self.mapping:
            _dict['mapping'] = self.mapping.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in data (list)
        _items = []
        if self.data:
            for _item in self.data:
                if _item:
                    _items.append(_item.to_dict())
            _dict['data'] = _items
        # set to None if data (nullable) is None
        # and __fields_set__ contains the field
        if self.data is None and "data" in self.__fields_set__:
            _dict['data'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TransactionsReconciliationsResponse:
        """Create an instance of TransactionsReconciliationsResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TransactionsReconciliationsResponse.parse_obj(obj)

        _obj = TransactionsReconciliationsResponse.parse_obj({
            "mapping": Mapping.from_dict(obj.get("mapping")) if obj.get("mapping") is not None else None,
            "data": [ReconciledTransaction.from_dict(_item) for _item in obj.get("data")] if obj.get("data") is not None else None
        })
        return _obj



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class PropertyDefinitionType(str, Enum):
    """
    PropertyDefinitionType
    """

    """
    allowed enum values
    """
    VALUEPROPERTY = 'ValueProperty'
    DERIVEDDEFINITION = 'DerivedDefinition'

    @classmethod
    def from_json(cls, json_str: str) -> PropertyDefinitionType:
        """Create an instance of PropertyDefinitionType from a JSON string"""
        return PropertyDefinitionType(json.loads(json_str))
